@@ -1,5 +1,5 @@
 # Miro - an RSS based video player application
-# Copyright (C) 2005-2009 Participatory Culture Foundation
+# Copyright (C) 2005-2010 Participatory Culture Foundation
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -79,6 +79,9 @@ class BaseTextEntry(SizedControl):
         self.create_signal('changed')
         self.create_signal('validate')
 
+    def focus(self):
+        self.view.window().makeFirstResponder_(self.view)
+
     def viewport_created(self):
         SizedControl.viewport_created(self)
         self.notifications.connect(self.on_changed, 'NSControlTextDidChangeNotification')
@@ -146,7 +149,61 @@ class SearchTextEntry(BaseTextEntry):
     def make_view(self):
         return MiroSearchTextField.alloc().init()
 
+class MultilineTextEntry(Widget):
 
+    def __init__(self, initial_text=None):
+        Widget.__init__(self)
+        if initial_text is None:
+            initial_text = ""
+        self.view = NSTextView.alloc().initWithFrame_(NSRect((0,0),(50,50)))
+        self.view.setMaxSize_((1.0e7, 1.0e7))
+        self.view.setHorizontallyResizable_(NO)
+        self.view.setVerticallyResizable_(YES)
+        self.notifications = NotificationForwarder.create(self.view)
+        if initial_text is not None:
+            self.set_text(initial_text)
+        self.set_size(widgetconst.SIZE_NORMAL)
+
+    def set_size(self, size):
+        if size == widgetconst.SIZE_NORMAL:
+            font = NSFont.systemFontOfSize_(NSFont.systemFontSize())
+        elif size == widgetconst.SIZE_SMALL:
+            self.view.cell().setControlSize_(NSSmallControlSize)
+        else:
+            raise ValueError("Unknown size: %s" % size)
+        self.view.setFont_(font)
+
+    def viewport_created(self):
+        Widget.viewport_created(self)
+        self.notifications.connect(self.on_changed, 'NSTextDidChangeNotification')
+        self.invalidate_size_request()
+
+    def remove_viewport(self):
+        Widget.remove_viewport(self)
+        self.notifications.disconnect()
+
+    def focus(self):
+        self.view.window().makeFirstResponder_(self.view)
+
+    def set_text(self, text):
+        self.view.setString_(text)
+        self.invalidate_size_request()
+
+    def get_text(self):
+        return self.view.string()
+
+    def on_changed(self, notification):
+        self.invalidate_size_request()
+
+    def calc_size_request(self):
+        if self.view.superview() is None:
+            return (50, 50)
+        width = self.view.superview().frame().size.width
+        height = self.view.frame().size.height
+        if self.parent_is_scroller:
+            width -= NSScroller.scrollerWidth()
+        return (width, height)
+    
 class MiroButton(NSButton):
     
     def initWithSignal_(self, signal):
@@ -263,7 +320,7 @@ class Button(SizedControl):
 class MiroPopupButton(NSPopUpButton):
 
     def init(self):
-        self = NSPopUpButton.init(self)
+        self = super(MiroPopupButton, self).init()
         self.setTarget_(self)
         self.setAction_('handleChange:')
         return self
@@ -387,7 +444,7 @@ class VideoSearchTextEntry (SearchTextEntry):
 class NSVideoSearchField (MiroSearchTextField):
 
     def init(self):
-        self = MiroSearchTextField.init(self)
+        self = super(NSVideoSearchField, self).init()
         self._engineToMenuItem = {}
         self.currentItem = nil
         self.setTarget_(self)
