@@ -1,5 +1,5 @@
 # Miro - an RSS based video player application
-# Copyright (C) 2005-2009 Participatory Culture Foundation
+# Copyright (C) 2005-2010 Participatory Culture Foundation
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -47,13 +47,6 @@ from miro.plat.frontends.widgets.drawing import DrawingContext, DrawingStyle, Gr
 from miro.plat.frontends.widgets.helpers import NotificationForwarder
 from miro.plat.frontends.widgets.layoutmanager import LayoutManager
 
-def get_all_indexes(tableview, index_set):
-    rows = list()
-    index = index_set.firstIndex()
-    while (index != NSNotFound):
-        rows.append(index)
-        index = index_set.indexGreaterThanIndex_(index)
-    return rows
 
 # Disclosure button used as a reference in get_left_offset()
 _disclosure_button = NSButton.alloc().init()
@@ -144,7 +137,7 @@ class HotspotTracker(object):
 
 class MiroTableCell(NSTextFieldCell):
     def init(self):
-        return self.initTextCell_('')
+        return super(MiroTableCell, self).initTextCell_('')
 
     def calcHeight_(self, view):
         font = self.font()
@@ -173,7 +166,7 @@ class MiroTableImageCell(NSImageCell):
 
 class MiroCheckboxCell(NSButtonCell):
     def init(self):
-        self = NSButtonCell.init(self)
+        self = super(MiroCheckboxCell, self).init()
         self.setButtonType_(NSSwitchButton)
         self.setTitle_('')
         return self
@@ -256,7 +249,7 @@ class CheckboxCellRenderer(signals.SignalEmitter):
 
 class CustomTableCell(NSCell):
     def init(self):
-        self = NSCell.init(self)
+        self = super(CustomTableCell, self).init()
         self.layout_manager = LayoutManager()
         self.hotspot = None
         return self
@@ -401,7 +394,7 @@ class VariableHeightOutlineViewDelegate(OutlineViewDelegate):
 
 class TableViewCommon(object):
     def init(self):
-        self = self.SuperClass.init(self)
+        self = super(self.__class__, self).init()
         self.hotspot_tracker = None
         self._tracking_rects = []
         self.hover_info = None
@@ -434,7 +427,7 @@ class TableViewCommon(object):
                         rect)
             context = NSGraphicsContext.currentContext()
             focused = self.isDescendantOf_(self.window().firstResponder())
-            for row in get_all_indexes(self, self.selectedRowIndexes()):
+            for row in tablemodel.list_from_nsindexset(self.selectedRowIndexes()):
                 self.drawBackgroundGradient(context, focused, row)
     
     def setFrameSize_(self, size):
@@ -1013,6 +1006,9 @@ class TableView(Widget):
             self._add_views()
         self.invalidate_size_request()
         self.queue_redraw()
+    
+    def is_showing_headers(self):
+        return self._show_headers
 
     def set_search_column(self, model_index):
         pass
@@ -1055,7 +1051,7 @@ class TableView(Widget):
     def get_selection(self):
         selection = self.tableview.selectedRowIndexes()
         return [self.model.iter_for_row(self.tableview, row)  \
-                for row in get_all_indexes(self.tableview, selection)]
+                for row in tablemodel.list_from_nsindexset(selection)]
 
     def get_selected(self):
         if self.tableview.allowsMultipleSelection():
