@@ -1,5 +1,5 @@
 # Miro - an RSS based video player application
-# Copyright (C) 2005-2009 Participatory Culture Foundation
+# Copyright (C) 2005-2010 Participatory Culture Foundation
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -34,13 +34,12 @@ import sys
 
 from miro.gtcache import gettext as _
 
-try:
-    import miro.libtorrent as lt
-except ImportError:
-    import libtorrent as lt
+import libtorrent as lt
 from miro.clock import clock
-from miro.download_utils import cleanFilename, nextFreeFilename, checkFilenameExtension, filterDirectoryName
-from miro.download_utils import filenameFromURL, getFileURLPath
+from miro.download_utils import (clean_filename, next_free_filename, 
+                                 check_filename_extension, 
+                                 filter_directory_name,
+                                 filename_from_url, get_file_url_path)
 from miro import eventloop
 from miro import httpclient
 import datetime
@@ -51,7 +50,7 @@ from miro import config
 from miro import prefs
 
 from miro.dl_daemon import command, daemon
-from miro.util import checkF, checkU, stringify, MAX_TORRENT_SIZE
+from miro.util import check_f, check_u, stringify, MAX_TORRENT_SIZE
 from miro.plat.utils import get_available_bytes_for_movies, utf8_to_filename
 
 chatter = True
@@ -65,8 +64,8 @@ def configReceived():
     torrentSession.startup()
 
 def createDownloader(url, contentType, dlid):
-    checkU(url)
-    checkU(contentType)
+    check_u(url)
+    check_u(contentType)
     if contentType == u'application/x-bittorrent':
         return BTDownloader(url, dlid)
     else:
@@ -74,10 +73,10 @@ def createDownloader(url, contentType, dlid):
 
 # Creates a new downloader object. Returns id on success, None on failure
 def startNewDownload(url, dlid, contentType, channelName):
-    checkU(url)
-    checkU(contentType)
+    check_u(url)
+    check_u(contentType)
     if channelName:
-        checkF(channelName)
+        check_f(channelName)
     dl = createDownloader(url, contentType, dlid)
     dl.channelName = channelName
     _downloads[dlid] = dl
@@ -122,7 +121,7 @@ def stopDownload(dlid, delete):
         return True
     return download.stop(delete)
 
-def stopUpload(dlid):
+def stop_upload(dlid):
     try:
         _lock.acquire()
         try:
@@ -136,9 +135,9 @@ def stopUpload(dlid):
         raise
     except: # There is no download with this id
         return
-    return download.stopUpload()
+    return download.stop_upload()
 
-def pauseUpload(dlid):
+def pause_upload(dlid):
     try:
         _lock.acquire()
         try:
@@ -152,10 +151,10 @@ def pauseUpload(dlid):
         raise
     except: # There is no download with this id
         return
-    return download.pauseUpload()
+    return download.pause_upload()
 
 def migrateDownload(dlid, directory):
-    checkF(directory)
+    check_f(directory)
     try:
         download = _downloads[dlid]
     except (SystemExit, KeyboardInterrupt):
@@ -185,6 +184,11 @@ def shutDown():
     torrentSession.shutdown()
 
 def restoreDownloader(downloader):
+    if downloader['dlid'] in _downloads:
+        logging.warn("Not restarting active downloader: %s",
+                downloader['dlid'])
+        return
+
     downloader = copy(downloader)
     dlerType = downloader.get('dlerType')
     if dlerType == u'HTTP':
@@ -335,9 +339,7 @@ class DownloadStatusUpdater:
     happen often enough to cause CPU problems.
     """
 
-    # setting this to 2 to prevent the "downloader playing catch-up"
-    # issue where suddenly a bunch of updates happen all at once.
-    UPDATE_CLIENT_INTERVAL = 2
+    UPDATE_CLIENT_INTERVAL = 1
 
     def __init__(self):
         self.toUpdate = set()
@@ -382,7 +384,7 @@ class BGDownloader:
         self.url = url
         self.startTime = clock()
         self.endTime = self.startTime
-        self.shortFilename = filenameFromURL(url)
+        self.shortFilename = filename_from_url(url)
         self.pickInitialFilename()
         self.state = u"downloading"
         self.currentSize = 0
@@ -401,8 +403,8 @@ class BGDownloader:
             'state': self.state,
             'totalSize': self.totalSize,
             'currentSize': self.currentSize,
-            'eta': self.getETA(),
-            'rate': self.getRate(),
+            'eta': self.get_eta(),
+            'rate': self.get_rate(),
             'uploaded': 0,
             'filename': self.filename,
             'startTime': self.startTime,
@@ -440,8 +442,8 @@ class BGDownloader:
         filename = self.shortFilename + suffix
         if not torrent:
             # this is an ascii filename and needs to be fixed
-            filename = cleanFilename(filename)
-        self.filename = nextFreeFilename(os.path.join(downloadDir, filename))
+            filename = clean_filename(filename)
+        self.filename = next_free_filename(os.path.join(downloadDir, filename))
 
     def moveToMoviesDirectory(self):
         """Move our downloaded file from the Incomplete Downloads directory to
@@ -452,11 +454,11 @@ class BGDownloader:
         self.moveToDirectory(config.get(prefs.MOVIES_DIRECTORY))
 
     def moveToDirectory(self, directory):
-        checkF(directory)
+        check_f(directory)
         if self.channelName:
-            channelName = filterDirectoryName(self.channelName)
-            # bug 10769: shutil and windows has problems with long filenames, so we clip
-            # the directory name.
+            channelName = filter_directory_name(self.channelName)
+            # bug 10769: shutil and windows has problems with long
+            # filenames, so we clip the directory name.
             if len(channelName) > 80:
                 channelName = channelName[:80]
             directory = os.path.join(directory, channelName)
@@ -469,18 +471,18 @@ class BGDownloader:
         newfilename = os.path.join(directory, self.shortFilename)
         if newfilename == self.filename:
             return
-        newfilename = nextFreeFilename(newfilename)
+        newfilename = next_free_filename(newfilename)
         def callback():
             self.filename = newfilename
             self.updateClient()
         fileutil.migrate_file(self.filename, newfilename, callback)
 
-    ##
-    # Returns a float with the estimated number of seconds left
-    def getETA(self):
+    def get_eta(self):
+        """Returns a float with the estimated number of seconds left.
+        """
         if self.totalSize == -1:
             return -1
-        rate = self.getRate()
+        rate = self.get_rate()
         if rate > 0:
             return (self.totalSize - self.currentSize) / rate
         else:
@@ -488,7 +490,7 @@ class BGDownloader:
 
     ##
     # Returns a float with the download rate in bytes per second
-    def getRate(self):
+    def get_rate(self):
         now = clock()
         if self.endTime != self.startTime:
             rate = self.currentSize/(self.endTime-self.startTime)
@@ -670,11 +672,11 @@ class HTTPDownloader(BGDownloader):
         #We have a success
         self.retryCount = -1
         #Get the length of the file, then create it
-        self.shortFilename = cleanFilename(info['filename'])
+        self.shortFilename = clean_filename(info['filename'])
         if self.expectedContentType is not None:
-            self.shortFilename = checkFilenameExtension(self.shortFilename, self.expectedContentType)
+            self.shortFilename = check_filename_extension(self.shortFilename, self.expectedContentType)
         else:
-            self.shortFilename = checkFilenameExtension(self.shortFilename, info.get('content-type'))
+            self.shortFilename = check_filename_extension(self.shortFilename, info.get('content-type'))
         self.pickInitialFilename()
         try:
             self.filehandle = fileutil.open_file(self.filename,"w+b")
@@ -831,7 +833,7 @@ class HTTPDownloader(BGDownloader):
         self.state = "stopped"
         self.updateClient()
 
-    def stopUpload(self):
+    def stop_upload(self):
         # HTTP downloads never upload.
         pass
 
@@ -983,7 +985,7 @@ class BTDownloader(BGDownloader):
         if config.get(prefs.LIMIT_UPLOAD_RATIO):
             if status.state == lt.torrent_status.states.seeding:
                 if float(self.uploaded)/self.totalSize > config.get(prefs.UPLOAD_RATIO):
-                    self.stopUpload()
+                    self.stop_upload()
 
     def handleError(self, shortReason, reason):
         self._shutdownTorrent()
@@ -1023,10 +1025,10 @@ class BTDownloader(BGDownloader):
         data['leechers'] = self.leechers
         return data
 
-    def getRate(self):
+    def get_rate(self):
         return self.rate
 
-    def getETA(self):
+    def get_eta(self):
         return self.eta
         
     def pause(self):
@@ -1050,12 +1052,12 @@ class BTDownloader(BGDownloader):
             except:
                 pass
 
-    def stopUpload(self):
+    def stop_upload(self):
         self.state = "finished"
         self._shutdownTorrent()
         self.updateClient()
 
-    def pauseUpload(self):
+    def pause_upload(self):
         self.state = "uploading-paused"
         self._shutdownTorrent()
         self.updateClient()
@@ -1131,7 +1133,7 @@ class BTDownloader(BGDownloader):
     def getMetainfo(self):
         if self.metainfo is None:
             if self.url.startswith('file://'):
-                path = getFileURLPath(self.url)
+                path = get_file_url_path(self.url)
                 try:
                     metainfoFile = open(path, 'rb')
                 except IOError:
