@@ -1,5 +1,5 @@
 # Miro - an RSS based video player application
-# Copyright (C) 2005-2009 Participatory Culture Foundation
+# Copyright (C) 2005-2010 Participatory Culture Foundation
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -31,7 +31,6 @@
 from miro import app
 from miro import signals
 from miro import messages
-from miro import menubar
 from miro.gtcache import gettext as _
 from miro.plat import resources
 from miro.frontends.widgets import style
@@ -39,8 +38,9 @@ from miro.frontends.widgets import separator
 from miro.frontends.widgets import imagepool
 from miro.frontends.widgets import statictabs
 from miro.frontends.widgets import widgetutil
+from miro.frontends.widgets import menus
 from miro.plat.frontends.widgets import widgetset
-
+from miro.plat.frontends.widgets import timer
 
 def send_new_order():
     def append_items(sequence, type):
@@ -58,10 +58,17 @@ def send_new_order():
 
 
 class TabListView(widgetset.TableView):
-    def __init__(self, renderer):
-        widgetset.TableView.__init__(self, 
-                widgetset.TreeTableModel('object'))
-        self.column = widgetset.TableColumn('tab', renderer, data=0)
+    def __init__(self, renderer, table_model_class=None):
+        if table_model_class is None:
+            table_model_class = widgetset.TreeTableModel
+        table_model = table_model_class('object', 'boolean', 'integer')
+        # columns are:
+        # - the tab_info object
+        # - should the tab should be blinking?
+        # - should we draw an uploading icon?  -1: no, 0-7: frame to draw
+        widgetset.TableView.__init__(self, table_model)
+        self.column = widgetset.TableColumn('tab', renderer, data=0, blink=1,
+                updating_frame=2)
         self.column.set_min_width(renderer.MIN_WIDTH)
         self.add_column(self.column)
         self.set_show_headers(False)
@@ -70,49 +77,104 @@ class TabListView(widgetset.TableView):
         self.set_fixed_height(True)
         self.set_auto_resizes(True)
 
-class StaticTabList(object):
-    """Handles the static tabs (the tabs on top that are always the same)."""
+    def append_tab(self, tab_info):
+        return self.model.append(tab_info, False, -1)
+
+    def append_child_tab(self, parent_iter, tab_info):
+        return self.model.append_child(parent_iter, tab_info, False, -1)
+
+    def update_tab(self, iter, tab_info):
+        self.model.update_value(iter, 0, tab_info)
+
+    def blink_tab(self, iter):
+        self.model.update_value(iter, 1, True)
+
+    def unblink_tab(self, iter):
+        self.model.update_value(iter, 1, False)
+
+    def pulse_updating_image(self, iter):
+        frame = self.model[iter][2]
+        self.model.update_value(iter, 2, (frame + 1) % 12)
+        self.model_changed()
+
+    def stop_updating_image(self, iter):
+        self.model.update_value(iter, 2, -1)
+        self.model_changed()
+
+class TabBlinkerMixin(object):
+    def blink_tab(self, id):
+        self.view.blink_tab(self.iter_map[id])
+        timer.add(1, self._unblink_tab, id)
+
+    def _unblink_tab(self, id):
+        # double check that the tab still exists
+        if id in self.iter_map:
+            self.view.unblink_tab(self.iter_map[id])
+
+class StaticTabListBase(TabBlinkerMixin):
+
     def __init__(self):
-        self.type = 'static'
-        self.view = TabListView(style.StaticTabRenderer())
-        self.view.allow_multiple_select(False)
-        self.view.set_fixed_height(False)
         self.iter_map = {}
         self.doing_change = False 
         # doing_change will be True if we are changing a bunch of tabs.  This
         # will cause us to not try to update things based on the selection
         # changing.
 
-    def build_tabs(self):
-        self.add(statictabs.ChannelGuideTab())
-        self.add(statictabs.SearchTab())
-        self.add(statictabs.LibraryTab())
-        self.add(statictabs.NewVideosTab())
-        self.add(statictabs.IndividualDownloadsTab())
-        self.add(statictabs.DownloadsTab())
-        self.view.model_changed()
-
     def add(self, tab):
-        iter = self.view.model.append(tab)
+        iter = self.view.append_tab(tab)
         self.iter_map[tab.id] = iter
 
-    def update_download_count(self, count):
-        iter = self.iter_map['downloading']
-        tab = self.view.model[iter][0]
-        tab.downloading = count
-        self.view.model.update(iter, tab)
-        self.view.model_changed()
-
-    def update_new_count(self, count):
-        iter = self.iter_map['new']
-        tab = self.view.model[iter][0]
-        tab.unwatched = count
-        self.view.model.update(iter, tab)
-        self.view.model_changed()
-
     def get_tab(self, name):
         return self.view.model[self.iter_map[name]][0]
 
+class StaticTabList(StaticTabListBase):
+    """Handles the static tabs (the tabs on top that are always the same)."""
+    def __init__(self):
+        StaticTabListBase.__init__(self)
+        self.type = 'static'
+        self.view = TabListView(style.StaticTabRenderer(),
+                widgetset.TableModel)
+        self.view.allow_multiple_select(False)
+        self.view.set_fixed_height(False)
+
+    def build_tabs(self):
+        self.add(statictabs.ChannelGuideTab())
+        self.add(statictabs.SearchTab())
+        self.view.model_changed()
+
+class LibraryTabList(StaticTabListBase):
+    """Handles all Library related tabs - Video, Audio, Downloading..."""
+    def __init__(self):
+        StaticTabListBase.__init__(self)
+        self.type = 'library'
+        self.view = TabListView(style.StaticTabRenderer())
+        self.view.allow_multiple_select(False)
+        self.view.set_fixed_height(False)
+        self.view.set_drag_dest(MediaTypeDropHandler())
+
+    def build_tabs(self):
+        self.add(statictabs.VideoLibraryTab())
+        self.add(statictabs.AudioLibraryTab())
+        self.add(statictabs.OtherLibraryTab())
+        self.add(statictabs.DownloadsTab())
+        self.view.model_changed()
+
+    def update_download_count(self, count):
+        self.update_count('downloading', 'downloading', count)
+
+    def update_new_video_count(self, count):
+        self.update_count('videos', 'unwatched', count)
+
+    def update_new_audio_count(self, count):
+        self.update_count('audios', 'unwatched', count)
+    
+    def update_count(self, key, attr, count):
+        iter = self.iter_map[key]
+        tab = self.view.model[iter][0]
+        setattr(tab, attr, count)
+        self.view.update_tab(iter, tab)
+        self.view.model_changed()
+
 class TabListDragHandler(object):
     def allowed_actions(self):
         return widgetset.DRAG_ACTION_MOVE
@@ -200,6 +262,37 @@ class TabDnDReorder(object):
                 retval[child_row[0].id] = child_iter
         return retval
 
+class MediaTypeDropHandler(object):
+    """Drop Handler that changes the media type (audio/video/other) of items
+    that get dropped on it.
+    """
+
+    def allowed_types(self):
+        return ('downloaded-item',)
+
+    def allowed_actions(self):
+        return widgetset.DRAG_ACTION_COPY
+
+    def validate_drop(self, table_view, model, type, source_actions, parent,
+            position):
+        if (type == 'downloaded-item' and parent is not None
+            and position == -1):
+            return widgetset.DRAG_ACTION_COPY
+        return widgetset.DRAG_ACTION_NONE
+
+    def accept_drop(self, table_view, model, type, source_actions, parent,
+            position, data):
+        if type == 'downloaded-item':
+            if parent is not None and position == -1:
+                video_ids = [int(id) for id in data.split('-')]
+                media_type = model[parent][0].media_type
+                m = messages.SetItemMediaType(media_type, video_ids)
+                m.send_to_backend()
+                return True
+        # We shouldn't get here, because don't allow it in validate_drop.
+        # Return False just in case
+        return False
+
 class TabListDropHandler(object):
     def __init__(self, tablist):
         self.tablist = tablist
@@ -228,6 +321,8 @@ class TabListDropHandler(object):
                 ((position == -1 and is_folder) or parent is not None)):
             # Don't allow folders to be dropped in other folders
             return widgetset.DRAG_ACTION_NONE
+        elif type not in self.item_types + self.folder_types:
+            return widgetset.DRAG_ACTION_NONE
         return widgetset.DRAG_ACTION_MOVE
 
     def accept_drop(self, table_view, model, type, source_actions, parent,
@@ -335,7 +430,7 @@ class PlaylistListDragHandler(TabListDragHandler):
     item_type = 'playlist'
     folder_type = 'playlist-with-folder'
 
-class TabList(signals.SignalEmitter):
+class TabList(signals.SignalEmitter, TabBlinkerMixin):
     """Handles a list of tabs on the left-side of Miro.
 
     signals:
@@ -381,7 +476,7 @@ class TabList(signals.SignalEmitter):
         self.doing_change = False
 
     def on_key_press(self, view, key, mods):
-        if key == menubar.DELETE:
+        if key == menus.DELETE:
             self.on_delete_key_pressed()
             return True
 
@@ -394,9 +489,9 @@ class TabList(signals.SignalEmitter):
         self.init_info(info)
         if parent_id:
             parent_iter = self.iter_map[parent_id]
-            iter = self.view.model.append_child(parent_iter, info)
+            iter = self.view.append_child_tab(parent_iter, info)
         else:
-            iter = self.view.model.append(info)
+            iter = self.view.append_tab(info)
         self.iter_map[info.id] = iter
 
     def set_folder_expanded(self, id, expanded):
@@ -405,7 +500,7 @@ class TabList(signals.SignalEmitter):
     def update(self, info):
         self.init_info(info)
         old_name = self.view.model[self.iter_map[info.id]][0].name
-        self.view.model.update(self.iter_map[info.id], info)
+        self.view.update_tab(self.iter_map[info.id], info)
         if old_name != info.name:
             self.emit('tab-name-changed', old_name, info.name)
 
@@ -455,7 +550,7 @@ class TabList(signals.SignalEmitter):
 class SiteList(TabList):
     type = 'site'
 
-    ALLOW_MULTIPLE = False
+    ALLOW_MULTIPLE = True
 
     def on_delete_key_pressed(self):
         app.widgetapp.remove_current_site()
@@ -473,11 +568,18 @@ class SiteList(TabList):
         info.unwatched = info.available = 0
 
     def on_context_menu(self, table_view):
-        return [
-            (_('Copy URL to clipboard'), app.widgetapp.copy_site_url),
-            (_('Rename Site'), app.widgetapp.rename_something),
-            (_('Remove Site'), app.widgetapp.remove_current_site),
-        ]
+        selected_rows = [table_view.model[iter][0] for iter in \
+                table_view.get_selection()]
+        if len(selected_rows) == 1:
+            return [
+                (_('Copy URL to clipboard'), app.widgetapp.copy_site_url),
+                (_('Rename Website'), app.widgetapp.rename_something),
+                (_('Remove Website'), app.widgetapp.remove_current_site),
+            ]
+        else:
+            return [
+                (_('Remove Websites'), app.widgetapp.remove_current_site),
+            ]
 
 class NestedTabList(TabList):
     """Tablist for tabs that can be put into folders (playlists and feeds)."""
@@ -498,14 +600,46 @@ class FeedList(NestedTabList):
 
     def __init__(self):
         TabList.__init__(self)
+        self.setup_dnd()
+        self.updating_animations = {}
+
+    def setup_dnd(self):
         self.view.set_drag_source(FeedListDragHandler())
         self.view.set_drag_dest(FeedListDropHandler(self))
 
     def on_delete_key_pressed(self):
         app.widgetapp.remove_current_feed()
 
+    def feed_is_updating(self, info):
+        if info.id in self.updating_animations:
+            return
+        timer_id = timer.add(0, self.pulse_updating_animation, info.id)
+        self.updating_animations[info.id] = timer_id
+
+    def feed_not_updating(self, info):
+        if info.id not in self.updating_animations:
+            return
+        self.view.stop_updating_image(self.iter_map[info.id])
+        timer_id = self.updating_animations.pop(info.id)
+        timer.cancel(timer_id)
+
+    def pulse_updating_animation(self, id):
+        try:
+            iter = self.iter_map[id]
+        except KeyError:
+            # feed was removed
+            del self.updating_animations[id]
+            return
+        self.view.pulse_updating_image(iter)
+        timer_id = timer.add(0.1, self.pulse_updating_animation, id)
+        self.updating_animations[id] = timer_id
+
     def init_info(self, info):
         info.icon = imagepool.get_surface(info.tab_icon, size=(16, 16))
+        if info.is_updating:
+            self.feed_is_updating(info)
+        else:
+            self.feed_not_updating(info)
 
     def get_feeds(self):
         infos = [self.view.model[i][0] for i in self.iter_map.values()]
@@ -544,8 +678,7 @@ class FeedList(NestedTabList):
         ]
 
 class AudioFeedList(FeedList):
-    def __init__(self):
-        TabList.__init__(self)
+    def setup_dnd(self):
         self.view.set_drag_source(AudioFeedListDragHandler())
         self.view.set_drag_dest(FeedListDropHandler(self))
 
@@ -598,15 +731,18 @@ class TabListBox(widgetset.Scroller):
         background.set_background_color((style.TAB_LIST_BACKGROUND_COLOR))
         background.add(self.build_vbox())
         self.add(background)
+        self.set_background_color((style.TAB_LIST_BACKGROUND_COLOR))
 
     def build_vbox(self):
         tlm = app.tab_list_manager
-        self.header_left_pad = tlm.feed_list.view.get_left_offset()
+        self.header_left_pad = 10
         vbox = widgetset.VBox()
         vbox.pack_start(tlm.static_tab_list.view)
-        vbox.pack_start(self.build_header(_('SITES')))
+        vbox.pack_start(self.build_header(_('LIBRARY')))
+        vbox.pack_start(tlm.library_tab_list.view)
+        vbox.pack_start(self.build_header(_('WEBSITES')))
         vbox.pack_start(tlm.site_list.view)
-        vbox.pack_start(self.build_header(_('FEEDS')))
+        vbox.pack_start(self.build_header(_('VIDEO FEEDS')))
         vbox.pack_start(tlm.feed_list.view)
         vbox.pack_start(self.build_header(_('AUDIO FEEDS')))
         vbox.pack_start(tlm.audio_feed_list.view)
@@ -615,8 +751,6 @@ class TabListBox(widgetset.Scroller):
         return vbox
 
     def build_header(self, text):
-        # hsep = separator.HThinSeparator(style.TAB_LIST_SEPARATOR_COLOR)
-
         label = widgetset.Label(text)
         label.set_bold(True)
         label.set_size(0.85)
