@@ -1,5 +1,5 @@
 # Miro - an RSS based video player application
-# Copyright (C) 2005-2009 Participatory Culture Foundation
+# Copyright (C) 2005-2010 Participatory Culture Foundation
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -35,7 +35,7 @@ from miro import feed
 from miro import folder
 from miro import eventloop
 from miro import playlist
-from miro.util import checkU
+from miro.util import check_u
 
 import logging
 
@@ -49,7 +49,7 @@ class TabOrder(database.DDBObject):
         """Construct a TabOrder.  type should be either ``channel`` or
         ``playlist``.
         """
-        checkU(type)
+        check_u(type)
         self.type = type
         self.tab_ids = []
         self._setup_views()
@@ -140,10 +140,10 @@ class TabOrder(database.DDBObject):
     def _check_for_non_existent_ids(self):
         changed = False
         for i in reversed(xrange(len(self.tab_ids))):
-            id = self.tab_ids[i]
-            if not id in self.id_to_tab:
+            tab_id = self.tab_ids[i]
+            if not tab_id in self.id_to_tab:
                 del self.tab_ids[i]
-                logging.warn("Throwing away non-existent TabOrder id: %s", id)
+                logging.warn("Throwing away non-existent TabOrder id: %s", tab_id)
                 changed = True
         if changed:
             self.signal_change()
@@ -208,7 +208,7 @@ class TabOrder(database.DDBObject):
         """Get all the tabs in this tab ordering (in order), regardless if
         they are visible in the tab list or not.
         """
-        return [self.id_to_tab[id] for id in self.tab_ids]
+        return [self.id_to_tab[tab_id] for tab_id in self.tab_ids]
 
     def _on_add_tab(self, tracker, obj):
         if obj.id not in self.id_to_tab:
@@ -232,7 +232,8 @@ class TabOrder(database.DDBObject):
         """Move a sequence of tabs so that they are after another tab."""
         anchor_pos = self.tab_ids.index(anchor_id)
         move_set = set(tab_ids)
-        before = [id for id in self.tab_ids[:anchor_pos] if id not in move_set]
-        after = [id for id in self.tab_ids[anchor_pos+1:] if id not in
-                move_set]
+        before = [tab_id for tab_id in self.tab_ids[:anchor_pos]
+                  if tab_id not in move_set]
+        after = [tab_id for tab_id in self.tab_ids[anchor_pos+1:]
+                 if tab_id not in move_set]
         self.reorder(before + [anchor_id] + list(tab_ids) + after)
