@@ -1,5 +1,8 @@
+import logging
+
 from miro.test.framework import MiroTestCase
 from miro import database
+from miro import databaselog
 from miro import item
 from miro import feed
 from miro import schema
@@ -8,18 +11,19 @@ class DatabaseTestCase(MiroTestCase):
     def setUp(self):
         MiroTestCase.setUp(self)
         self.feed = feed.Feed(u"http://feed.org")
-        self.i1 = item.Item({'title': u'item1'},
-                       feed_id=self.feed.id)
-        self.i2 = item.Item({'title': u'item2'},
-                       feed_id=self.feed.id)
+        self.i1 = item.Item(item.FeedParserValues({'title': u'item1'}),
+                            feed_id=self.feed.id)
+        self.i2 = item.Item(item.FeedParserValues({'title': u'item2'}),
+                            feed_id=self.feed.id)
+
         self.feed2 = feed.Feed(u"http://feed.com")
-        self.i3 = item.Item({'title': u'item3'},
-                       feed_id=self.feed2.id)
+        self.i3 = item.Item(item.FeedParserValues({'title': u'item3'}),
+                            feed_id=self.feed2.id)
 
 class ViewTest(DatabaseTestCase):
     def test_iter(self):
         view = item.Item.make_view('feed_id=?', (self.feed.id,))
-        self.assertEquals(set(view), set([self.i2, self.i1]))
+        self.assertSameSet(view, [self.i2, self.i1])
 
     def test_count(self):
         view = item.Item.make_view('feed_id=?', (self.feed.id,))
@@ -29,7 +33,7 @@ class ViewTest(DatabaseTestCase):
         self.feed.set_title(u'booya')
         view = item.Item.make_view("feed.userTitle='booya'",
                 joins={'feed': 'feed.id=item.feed_id'})
-        self.assertEquals(set(view), set([self.i2, self.i1]))
+        self.assertSameSet(view, [self.i2, self.i1])
         self.assertEquals(view.count(), 2)
 
 class ViewTrackerTest(DatabaseTestCase):
@@ -86,44 +90,19 @@ class ViewTrackerTest(DatabaseTestCase):
         self.assertEquals(self.remove_callbacks, [self.feed, self.feed2])
         self.assertEquals(self.change_callbacks, [self.feed2])
 
-    def test_track_join(self):
-        self.setup_view(item.Item.make_view("feed.userTitle='booya'",
-                joins={'feed': 'feed.id=item.feed_id'}))
-        self.feed2.set_title(u'booya')
-        self.feed2.signal_related_change()
-        self.assertEquals(self.add_callbacks, [self.i3])
-        self.assertEquals(self.remove_callbacks, [])
-        # i1 and i2 are in the first feed, which is in our view.  They will
-        # get the changed signal.  Note: the order is not defined and could
-        # also be [i2, i1], but we don't worry about that.
-        self.assertEquals(self.change_callbacks, [self.i1, self.i2])
-        self.feed2.revert_title()
-        self.feed2.signal_related_change()
-        self.assertEquals(self.add_callbacks, [self.i3])
-        self.assertEquals(self.remove_callbacks, [self.i3])
-        self.assertEquals(self.change_callbacks, [self.i1, self.i2, self.i1,
-            self.i2])
-        # i1 and i2 should get the changed signal again.  i3 won't get it
-        # because it got the removed signal.
-
     def test_track_creation_add(self):
         self.setup_view(item.Item.make_view("feed.userTitle='booya'",
                 joins={'feed': 'feed.id=item.feed_id'}))
-        self.feed2.set_title(u'booya')
-        self.feed2.signal_related_change()
-        self.assertEquals(self.add_callbacks, [self.i3])
 
-        i4 = item.Item({'title': u'item4'}, feed_id=self.feed.id)
-        self.assertEquals(self.add_callbacks, [self.i3, i4])
+        i4 = item.Item(item.FeedParserValues({'title': u'item4'}), 
+                       feed_id=self.feed.id)
+        self.assertEquals(self.add_callbacks, [i4])
 
     def test_track_destruction_remove(self):
         self.setup_view(item.Item.make_view("feed.userTitle='booya'",
                 joins={'feed': 'feed.id=item.feed_id'}))
-        self.feed2.set_title(u'booya')
-        self.feed2.signal_related_change()
-        self.assertEquals(self.remove_callbacks, [])
-        self.i3.remove()
-        self.assertEquals(self.remove_callbacks, [self.i3])
+        self.i1.remove()
+        self.assertEquals(self.remove_callbacks, [self.i1])
 
     def test_unlink(self):
         self.tracker.unlink()
@@ -133,20 +112,8 @@ class ViewTrackerTest(DatabaseTestCase):
         self.assertEquals(self.remove_callbacks, [])
         self.assertEquals(self.change_callbacks, [])
 
-    def test_unlink_join(self):
-        self.setup_view(item.Item.make_view("feed.userTitle='booya'",
-                joins={'feed': 'feed.id=item.feed_id'}))
-        self.tracker.unlink()
-        self.feed2.set_title(u'booya')
-        self.feed2.signal_related_change()
-        self.feed.revert_title()
-        self.feed.signal_related_change()
-        self.assertEquals(self.add_callbacks, [])
-        self.assertEquals(self.remove_callbacks, [])
-        self.assertEquals(self.change_callbacks, [])
-
     def test_reset(self):
-        database.ViewTracker.reset_trackers()
+        database.setup_managers()
         self.feed2.set_title(u"booya")
         self.feed.revert_title()
         self.assertEquals(self.add_callbacks, [])
@@ -156,7 +123,7 @@ class ViewTrackerTest(DatabaseTestCase):
 
 class TestDDBObject(database.DDBObject):
     def setup_new(self, testcase, remove=False):
-        testcase.id_exists_retval = self.idExists()
+        testcase.id_exists_retval = self.id_exists()
         if remove:
             self.remove()
 
@@ -185,8 +152,67 @@ class DDBObjectTestCase(MiroTestCase):
 
     def test_test_attribute_track(self):
         testobj = TestDDBObject(self)
-        self.assertEquals(testobj.changed_attributes, set(['id']))
+        self.assertEquals(testobj.changed_attributes, set())
         testobj.foo = 1
-        self.assertEquals(testobj.changed_attributes, set(['id', 'foo']))
+        self.assertEquals(testobj.changed_attributes, set(['foo']))
         testobj.bar = 2
-        self.assertEquals(testobj.changed_attributes, set(['id', 'foo']))
+        self.assertEquals(testobj.changed_attributes, set(['foo']))
+
+class LogFilter(logging.Filter):
+    def __init__(self):
+        self.allow = False
+        self.records = []
+
+    def filter(self, record):
+        if not self.allow:
+            raise AssertionError("We shouldn't see any logging")
+        else:
+            self.records.append(record)
+            return False
+
+    def forget_records(self):
+        self.records = []
+
+class DatabaseLoggingTest(MiroTestCase):
+    def setUp(self):
+        MiroTestCase.setUp(self)
+        logger = logging.getLogger()
+        self._log_level = logger.getEffectiveLevel()
+        self._old_filters = logger.filters
+
+        self.log_filter = LogFilter()
+        logger.setLevel(logging.DEBUG)
+        for old_filter in logger.filters:
+            logger.removeFilter(old_filter)
+        logger.addFilter(self.log_filter)
+
+    def tearDown(self):
+        MiroTestCase.tearDown(self)
+        logger = logging.getLogger()
+        logger.setLevel(self._log_level)
+        for old_filter in logger.filters:
+            logger.removeFilter(old_filter)
+        for filter in self._old_filters:
+            logger.addFilter(filter)
+
+    def check_records(self, count):
+        records = self.log_filter.records
+        self.assertEqual(len(records), count)
+        for rec in records:
+            self.assertEqual(rec.levelno, logging.getLevelName('DBLOG'))
+
+    def test_warning_logged(self):
+        self.log_filter.allow = True
+        databaselog.info("message %s", 1)
+        databaselog.debug("message %s", 2)
+        self.check_records(2)
+
+    def test_backlog(self):
+        self.log_filter.allow = True
+        databaselog.info("message %s", 1)
+        databaselog.debug("message %s", 2)
+        self.log_filter.forget_records()
+        databaselog.print_old_log_entries()
+        # should have 3 log entries, 1 header, 1 footer, and the 1 for the
+        # info message
+        self.check_records(3)
