@@ -8,6 +8,7 @@ import time
 from miro import app
 from miro import database
 from miro import databaseupgrade
+from miro import dialogs
 from miro import downloader
 from miro import item
 from miro import feed
@@ -15,22 +16,24 @@ from miro import folder
 from miro import frontendstate
 from miro import guide
 from miro import schema
+from miro import signals
 from miro import tabs
 from miro import theme
 import shutil
 from miro import storedatabase
 from miro.plat import resources
+from miro.plat.utils import FilenameType
 
 from miro.test.framework import MiroTestCase, EventLoopTest
-# sooo much easier to type...
-from miro.schema import SchemaString, SchemaInt, SchemaFloat, SchemaReprContainer
-from miro.schema import SchemaList, SchemaDict, SchemaObject, SchemaBool
-from miro.schema import SchemaFilename, SchemaBinary
+from miro.schema import (SchemaString, SchemaInt, SchemaFloat,
+                         SchemaReprContainer, SchemaList, SchemaDict,
+                         SchemaObject, SchemaBool, SchemaFilename,
+                         SchemaBinary)
 
 # create a dummy object schema
 class Human(database.DDBObject):
     def setup_new(self, name, age, meters_tall, friends, high_scores=None,
-            **stuff):
+                  **stuff):
         self.name = name
         self.age = age
         self.meters_tall = meters_tall
@@ -51,6 +54,12 @@ class RestorableHuman(Human):
     def setup_restored(self):
         self.iveBeenRestored = True
 
+class DBInsertCallbackHuman(Human):
+    callback = None
+    def on_db_insert(self):
+        if self.__class__.callback:
+            self.__class__.callback(self)
+
 class PCFProgramer(Human):
     def setup_new(self, name, age, meters_tall, friends, file, developer,
             high_scores = None):
@@ -61,7 +70,7 @@ class PCFProgramer(Human):
 class SpecialProgrammer(PCFProgramer):
     def setup_new(self):
         PCFProgramer.setup_new(self, u'I.M. Special', 44, 2.1, [],
-                '/home/specialdude/\u1234'.encode("utf-8"), True)
+                FilenameType('/home/specialdude/\u1234'.encode("utf-8")), True)
 
 class HumanSchema(schema.ObjectSchema):
     klass = Human
@@ -85,8 +94,11 @@ class RestorableHumanSchema(HumanSchema):
     klass = RestorableHuman
     table_name = 'restorable_human'
 
-class PCFProgramerSchema(schema.MultiClassObjectSchema):
+class DBInsertCallbackHumanSchema(HumanSchema):
+    klass = DBInsertCallbackHuman
+    table_name = 'db_insert_callback_human'
 
+class PCFProgramerSchema(schema.MultiClassObjectSchema):
     table_name = 'pcf_programmer'
     fields = HumanSchema.fields + [
         ('file', SchemaFilename()),
@@ -104,13 +116,14 @@ class PCFProgramerSchema(schema.MultiClassObjectSchema):
         else:
             return PCFProgramer
 
-test_object_schemas = [HumanSchema, PCFProgramerSchema, RestorableHumanSchema]
+test_object_schemas = [HumanSchema, PCFProgramerSchema, RestorableHumanSchema,
+        DBInsertCallbackHumanSchema]
 
 def upgrade1(cursor):
     cursor.execute("UPDATE human set name='new name'")
 
 def upgrade2(cursor):
-    1/0
+    1 / 0
 
 class StoreDatabaseTest(EventLoopTest):
     OBJECT_SCHEMAS = None
@@ -134,7 +147,7 @@ class StoreDatabaseTest(EventLoopTest):
     def tearDown(self):
         self.remove_database()
         corrupt_path = os.path.join(os.path.dirname(self.save_path),
-                'corrupt_database')
+                                    'corrupt_database')
         if os.path.exists(corrupt_path):
             os.remove(corrupt_path)
         databaseupgrade._upgrade_overide = {}
@@ -170,13 +183,13 @@ class DBUpgradeTest(StoreDatabaseTest):
         self.remove_database()
         self.reload_database()
         app.db.cursor.execute("SELECT name FROM sqlite_master "
-                "WHERE type='index'")
+                              "WHERE type='index'")
         blank_db_indexes = set(app.db.cursor)
         shutil.copy(resources.path("testdata/olddatabase.v79"),
-                self.save_path2)
+                    self.save_path2)
         self.reload_database(self.save_path2)
         app.db.cursor.execute("SELECT name FROM sqlite_master "
-                "WHERE type='index'")
+                              "WHERE type='index'")
         upgraded_db_indexes = set(app.db.cursor)
         self.assertEquals(upgraded_db_indexes, blank_db_indexes)
 
@@ -185,31 +198,21 @@ class DBUpgradeTest(StoreDatabaseTest):
         self.reload_database()
         blank_column_types = self._get_column_types()
         shutil.copy(resources.path("testdata/olddatabase.v79"),
-                self.save_path2)
+                    self.save_path2)
         self.reload_database(self.save_path2)
         upgraded_column_types = self._get_column_types()
         self.assertEquals(set(blank_column_types.keys()),
-                set(upgraded_column_types.keys()))
+                          set(upgraded_column_types.keys()))
         for table_name in blank_column_types:
             diff = blank_column_types[table_name].symmetric_difference(
-                    upgraded_column_types[table_name])
+                upgraded_column_types[table_name])
             if diff:
                 raise AssertionError("different column types for %s (%s)" %
-                        (table_name, diff))
-
-    def test_empty_feedparser_output(self):
-        # make sure that if we reset feedparser_output to {} because of bug
-        # #12028 that our upgrades work.  The item with id 44 in this database
-        # has feedparser_output = {}
-        shutil.copy(resources.path("testdata/"
-            "olddatabase.empty_feedparser_output_test"), self.save_path2)
-        self.reload_database(self.save_path2)
-        app.db.cursor.execute('select count(*) from item')
-        self.assertEquals(item.Item.make_view().count(), 15)
+                                     (table_name, diff))
 
     def _get_column_types(self):
         app.db.cursor.execute("SELECT name FROM sqlite_master "
-                "WHERE type='table'")
+                              "WHERE type='table'")
         rv = {}
         for table_name in app.db.cursor.fetchall():
             app.db.cursor.execute('pragma table_info(%s)' % table_name)
@@ -223,9 +226,9 @@ class FakeSchemaTest(StoreDatabaseTest):
         StoreDatabaseTest.setUp(self)
         self.lee = Human(u"lee", 25, 1.4, [], {u'virtual bowling': 212})
         self.joe = RestorableHuman(u"joe", 14, 1.4, [self.lee], car=u'toyota',
-                dog=u'scruffy')
+                                   dog=u'scruffy')
         self.ben = PCFProgramer(u'ben', 25, 3.4, [self.joe],
-                '/home/ben/\u1234'.encode("utf-8"), True)
+                                FilenameType('/home/ben/\u1234'.encode("utf-8")), True)
         self.db = [self.lee, self.joe, self.ben]
         databaseupgrade._upgrade_overide[1] = upgrade1
         databaseupgrade._upgrade_overide[2] = upgrade2
@@ -256,7 +259,8 @@ class DiskTest(FakeSchemaTest):
                         obj_value, name))
 
     def test_create(self):
-        # Test that the database we set up in __init__ restores correctly
+        # Test that the database we set up in __init__ restores
+        # correctly
         self.reload_test_database()
         self.check_database()
 
@@ -316,13 +320,12 @@ class DiskTest(FakeSchemaTest):
         self.check_database()
 
     def test_commit_without_close(self):
-        # we should commit using an idle callback.
-        self.runPendingIdles()
+        app.db.finish_transaction()
         # close the database connection without giving LiveStorage the
         # oppertunity to commit when it's closed.
         app.db.connection.close()
-        app.db = storedatabase.LiveStorage(self.save_path,
-                schema_version=0, object_schemas=self.OBJECT_SCHEMAS)
+        self.setup_new_database(self.save_path, schema_version=0,
+                object_schemas=self.OBJECT_SCHEMAS)
         app.db.upgrade_database()
         self.check_database()
 
@@ -345,23 +348,88 @@ class DiskTest(FakeSchemaTest):
 
     def check_reload_error(self, **reload_args):
         corrupt_path = os.path.join(os.path.dirname(self.save_path),
-                'corrupt_database')
+                                    'corrupt_database')
         self.assert_(not os.path.exists(corrupt_path))
+        self.allow_db_load_errors(True)
         self.reload_test_database(**reload_args)
+        self.allow_db_load_errors(False)
         self.assert_(os.path.exists(corrupt_path))
 
+    def start_fresh_on_error_dialog(self):
+        def dialog_handler(obj, dialog):
+            dialog.run_callback(dialogs.BUTTON_START_FRESH)
+        signals.system.connect('new-dialog', dialog_handler)
+
     def test_upgrade_error(self):
+        self.start_fresh_on_error_dialog()
         self.check_reload_error(version=2)
 
     def test_corrupt_database(self):
         app.db.close()
         open(self.save_path, 'wb').write("BOGUS DATA")
+        self.start_fresh_on_error_dialog()
         self.check_reload_error()
 
     def test_database_data_error(self):
         app.db.cursor.execute("DROP TABLE human")
         self.check_reload_error()
 
+    def test_bulk_insert(self):
+        new_humans = []
+        app.bulk_sql_manager.start()
+        # nothing should be inserted yet
+        for x in range(10):
+            name = u"lee-clone-%s" % x
+            new_humans.append(Human(name, 25, 1.4, [], {u'virtual bowling':
+                                                        212}))
+        self.check_database()
+        self.assertEquals(Human.make_view().count(), 1)
+        app.bulk_sql_manager.finish()
+        # calling finish() should insert the new Humans
+        self.db.extend(new_humans)
+        self.check_database()
+        self.assertEquals(Human.make_view().count(), 11)
+
+    def test_bulk_remove(self):
+        new_humans = []
+        for x in range(10):
+            name = u"lee-clone-%s" % x
+            new_humans.append(Human(name, 25, 1.4, [], {u'virtual bowling':
+                                                        212}))
+        self.assertEquals(Human.make_view().count(), 11)
+        app.bulk_sql_manager.start()
+        for new_dude in new_humans:
+            new_dude.remove()
+        self.assertEquals(Human.make_view().count(), 11)
+        app.bulk_sql_manager.finish()
+        self.assertEquals(Human.make_view().count(), 1)
+
+    def test_insert_during_on_insert(self):
+        # what happens if on_db_insert creates a new item (see #12680)
+        def insert_callback(obj):
+            Human(u'minnie', 25, 1.4, [], {})
+        DBInsertCallbackHuman.callback = insert_callback
+        app.bulk_sql_manager.start()
+        ben = DBInsertCallbackHuman(u'janet', 25, 1.4, [], {})
+        # calling finish() will invoke insert_callback and create minnie.
+        # Check that this is reflected on disk
+        self.assertEquals(Human.make_view().count(), 1)
+        app.bulk_sql_manager.finish()
+        self.assertEquals(Human.make_view().count(), 2)
+
+    def test_bulk_insert_with_signal_change(self):
+        app.bulk_sql_manager.start()
+        # nothing should be inserted yet
+        lee2 = Human(u'lee2', 25, 1.4, [], {u'virtual bowling': 212})
+        lee2.name = u'lee2-changed'
+        # calling signal_change() shouldn't throw an exception like it did in
+        # ticket #12806
+        lee2.signal_change()
+        app.bulk_sql_manager.finish()
+        # double check that the new name is the correct one.
+        lee2 = self.reload_object(lee2)
+        self.assertEqual(lee2.name, u'lee2-changed')
+
 class ObjectMemoryTest(FakeSchemaTest):
     def test_remove_remove_object_map(self):
         self.reload_test_database()
@@ -385,40 +453,40 @@ class ValidationTest(FakeSchemaTest):
     def assert_object_invalid(self, obj):
         self.assertRaises(schema.ValidationError, obj.signal_change)
 
-    def testNoneValues(self):
+    def test_none_values(self):
         self.lee.age = None
         self.assert_object_invalid(self.lee)
         self.lee.age = 25
         self.lee.stuff = None
         self.assert_object_valid(self.lee)
 
-    def testIntValidation(self):
+    def test_int_validation(self):
         self.lee.age = '25'
         self.assert_object_invalid(self.lee)
         self.lee.age = 25L
         self.assert_object_valid(self.lee)
 
-    def testStringValidation(self):
+    def test_string_validation(self):
         self.lee.name = 133
         self.assert_object_invalid(self.lee)
         self.lee.name = u'lee'
         self.assert_object_valid(self.lee)
 
-    def testBinaryValidation(self):
+    def test_binary_validation(self):
         self.lee.id_code = u'abc'
         self.assert_object_invalid(self.lee)
         self.lee.id_code = 'abc'
         self.assert_object_valid(self.lee)
 
-    def testFloatValidation(self):
+    def test_float_validation(self):
         self.lee.meters_tall = 3
         self.assert_object_invalid(self.lee)
 
-    def testListValidation(self):
+    def test_list_validation(self):
         self.lee.friend_names = [1234]
         self.assert_object_invalid(self.lee)
 
-    def testDictValidation(self):
+    def test_dict_validation(self):
         self.joe.high_scores['pong'] = u"One Million"
         self.assert_object_invalid(self.joe)
         del self.joe.high_scores['pong']
@@ -430,7 +498,7 @@ class CorruptReprTest(FakeSchemaTest):
     # (#12028)
     def test_repr_failure(self):
         app.db.cursor.execute("UPDATE human SET stuff='{baddata' "
-                "WHERE name='lee'")
+                              "WHERE name='lee'")
         restored_lee = self.reload_object(self.lee)
         self.assertEqual(restored_lee.stuff, 'testing123')
         app.db.cursor.execute("SELECT stuff from human WHERE name='lee'")
@@ -439,7 +507,7 @@ class CorruptReprTest(FakeSchemaTest):
 
     def test_repr_failure_no_handler(self):
         app.db.cursor.execute("UPDATE pcf_programmer SET stuff='{baddata' "
-                "WHERE name='ben'")
+                              "WHERE name='ben'")
         self.assertRaises(SyntaxError, self.reload_object, self.ben)
 
 class ConverterTest(StoreDatabaseTest):
@@ -448,18 +516,20 @@ class ConverterTest(StoreDatabaseTest):
 
         test1 = """{'updated_parsed': (2009, 6, 5, 1, 30, 0, 4, 156, 0)}"""
         val = converter._convert_repr(test1)
-        self.assertEquals(val, {"updated_parsed": (2009, 6, 5, 1, 30, 0, 4, 156, 0)})
+        self.assertEquals(val, {"updated_parsed":
+                                (2009, 6, 5, 1, 30, 0, 4, 156, 0)})
 
         test2 = """{'updated_parsed': time.struct_time(tm_year=2009, tm_mon=6, tm_mday=5, tm_hour=1, tm_min=30, tm_sec=0, tm_wday=4, tm_yday=156, tm_isdst=0)}"""
         val = converter._convert_repr(test2)
-        self.assertEquals(val, {"updated_parsed": (2009, 6, 5, 1, 30, 0, 4, 156, 0)})
+        self.assertEquals(val, {"updated_parsed":
+                                (2009, 6, 5, 1, 30, 0, 4, 156, 0)})
 
 class CorruptDDBObjectReprTest(StoreDatabaseTest):
     # test corrupt SchemaReprContainer columns in real DDBObjects
     def setUp(self):
         StoreDatabaseTest.setUp(self)
         self.feed = feed.Feed(u"dtv:multi:http://feed.org/,query")
-        self.item = item.Item({'title': u'item1'},
+        self.item = item.Item(item.FeedParserValues({'title': u'item1'}),
                        feed_id=self.feed.id)
         self.downloader = downloader.RemoteDownloader(
             u'http://example.com/1/item1/movie.mpeg', self.item)
@@ -489,16 +559,11 @@ class CorruptDDBObjectReprTest(StoreDatabaseTest):
     def test_corrupt_status(self):
         app.db.cursor.execute("UPDATE remote_downloader "
                 "SET status='{baddata' WHERE id=?", (self.downloader.id,))
-        # setup_restored sets some values for status, so we will have more
-        # than an empty dict
+        # setup_restored sets some values for status, so we will have
+        # more than an empty dict
         self.check_fixed_value(self.downloader, 'status',
                 {'rate': 0, 'upRate': 0, 'eta': 0}, disk_value={})
 
-    def test_corrupt_feedparser_output(self):
-        app.db.cursor.execute("UPDATE item "
-                "SET feedparser_output='{baddata' WHERE id=?", (self.item.id,))
-        self.check_fixed_value(self.item, 'feedparser_output', {})
-
     def test_corrupt_etag(self):
         app.db.cursor.execute("UPDATE rss_multi_feed_impl "
                 "SET etag='{baddata' WHERE ufeed_id=?", (self.feed.id,))
